SystemOrganization addCategory: #'Ice-Tests'!

TestResource subclass: #ICTestResource
	instanceVariableNames: 'fakeClass'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ice-Tests'!

!ICTestResource methodsFor: 'running' stamp: 'EstebanLorenzano 1/16/2011 18:01'!
setUp
	PackageOrganizer default 
		packageNamed: 'Unclassified' 
		ifAbsent: [ PackageOrganizer default registerPackageNamed: 'Unclassified' ].  
	PackageOrganizer default 
		packageNamed: 'FakeExtension' 
		ifAbsent: [ PackageOrganizer default registerPackageNamed: 'FakeExtension' ].  
	fakeClass := Class subclass: #FakeClass.  
	fakeClass 
		compileSilently: 'fakeMethod ^42' 
		classified: #'*fakeExtension'.! !

!ICTestResource methodsFor: 'running' stamp: 'EstebanLorenzano 1/16/2011 18:02'!
tearDown   
	PackageOrganizer default unregisterPackageNamed: 'FakeExtension'.
	fakeClass removeFromSystem: false.  ! !

TestCase subclass: #ICBaseTest
	instanceVariableNames: 'repository'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ice-Tests'!

!ICBaseTest classSide methodsFor: 'as yet unclassified' stamp: 'DeboraFortini 9/7/2011 12:20'!
resources 
	^Array with: ICTestResource! !

!ICBaseTest methodsFor: 'accessing' stamp: 'EstebanLorenzano 1/17/2011 13:19'!
fakeClass 
	^Smalltalk 
		at: #FakeClass 
		ifAbsent: [ nil ]! !

!ICBaseTest methodsFor: 'accessing' stamp: 'EstebanLorenzano 1/17/2011 13:21'!
repository 
	^repository! !

!ICBaseTest methodsFor: 'running' stamp: 'DeboraFortini 9/6/2011 17:01'!
setUp  
	repository := ICRepository on: (FSFilesystem memory).! !

ICBaseTest subclass: #ICChangeTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ice-Tests'!

!ICChangeTest methodsFor: 'tests' stamp: 'EstebanLorenzano 1/19/2011 10:50'!
testRemoveClassChange
	| newChange |
	
	(self repository storage / 'Unclassified' / 'FakeClass.st')
		in: [ :me | me parent ensureDirectory ];
		fileStreamDo: [ :stream | stream nextPutAll: 'test' ].  
	newChange := ICRemoveClassChange className: #FakeClass category: 'Unclassified'.
	newChange applyOn: self repository.
	
	self deny: (self repository storage / 'Unclassified' / 'FakeClass.st') exists. ! !

!ICChangeTest methodsFor: 'tests' stamp: 'EstebanLorenzano 1/19/2011 10:50'!
testUpdateClassChange
	| newChange |
	newChange := ICUpdateClassChange className: #FakeClass category: 'Unclassified'.
	newChange applyOn: self repository.
	
	self assert: (self repository storage / 'Unclassified' / 'FakeClass.st') exists. ! !

!ICChangeTest methodsFor: 'tests' stamp: 'EstebanLorenzano 1/19/2011 15:57'!
testUpdateExtensionsChange
	| newChange |
	
	self repository addPackageNamed: 'FakeExtension'.  
	newChange := ICUpdateExtensionsChange 
		className: #FakeClass 
		category: 'Extensions'
		packages: (self repository containers select: [ :each | each isExtension not ]).
	newChange applyOn: self repository.
	
	self assert: (self repository storage / 'Extensions' / 'FakeClass.st') exists. ! !

ICBaseTest subclass: #ICContainerTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ice-Tests'!

!ICContainerTest methodsFor: 'tests' stamp: 'EstebanLorenzano 1/17/2011 16:59'!
testActionAddClass
	| container |
	
	container := ICPackageContainer for: (PackageOrganizer default 
		packageNamed: 'Unclassified' 
		ifAbsent: [ self error ]).
	container addAction: (ICAddClassAction for: self fakeClass).
	self assert: container actions size = 1.
	self assert: container allActions size = 1.! !

!ICContainerTest methodsFor: 'tests' stamp: 'EstebanLorenzano 1/17/2011 17:03'!
testActionAddExtension
	| container |
	
	container := ICExtensionContainer for: (PackageOrganizer default 
		packageNamed: 'FakeExtension' 
		ifAbsent: [ self error ]).
	container addAction: (ICAddMethodAction for: (MethodReference 
		class: self fakeClass
		selector: #fakeMethod)).
	self assert: container actions size = 1.
	self assert: container allActions size = 1.! !

ICBaseTest subclass: #ICOperationTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ice-Tests'!

!ICOperationTest methodsFor: 'tests' stamp: 'EstebanLorenzano 1/19/2011 15:57'!
testPullClass
	| fakeClass2 |

	fakeClass2 := Class subclass: #FakeClass2.  
	self repository addPackageNamed: 'Unclassified'.
	self repository push.
	
	fakeClass2 removeFromSystem: false.
	self repository removeHistoryAt: #FakeClass2.
	  
	self repository pull.
	self assert: (Smalltalk at: #FakeClass2 ifAbsent: [ nil ]) notNil.
	(Smalltalk at: #FakeClass2) removeFromSystem: false.  
	! !

!ICOperationTest methodsFor: 'tests' stamp: 'EstebanLorenzano 1/19/2011 15:57'!
testPullExtension
	self repository addPackageNamed: 'FakeExtension'.
	self repository push.

	self fakeClass removeSelector: #fakeMethod.  
	self repository removeHistoryAt: #FakeClass.  	

	self repository pull.
	
	self assert: (self fakeClass selectors includes: #fakeMethod).
	! !

!ICOperationTest methodsFor: 'tests' stamp: 'EstebanLorenzano 1/19/2011 15:57'!
testPushAddClass
	self repository addPackageNamed: 'Unclassified'.
	self repository push.
	
	self assert: (self repository storage / 'Unclassified' / 'FakeClass.st') exists. ! !

!ICOperationTest methodsFor: 'tests' stamp: 'EstebanLorenzano 1/19/2011 15:57'!
testPushExtensions
	self repository addPackageNamed: 'FakeExtension'.
	self repository push.
	
	self assert: (self repository storage / 'Extensions' / 'FakeClass.st') exists. 
	self deny: (self repository storage / 'Unclassified' / 'FakeClass.st') exists. ! !

!ICOperationTest methodsFor: 'tests' stamp: 'EstebanLorenzano 1/19/2011 12:43'!
testPushManyActions
	"Tests many actions. 
	  WARNING I'm using Kernel package to check the correct push, but this can change any time"
	self repository containers add: (ICPackageContainer for: (PackageOrganizer default 
		packageNamed: 'Kernel'
		ifAbsent: [ self error ])).
	self repository containers add: (ICPackageContainer for: (PackageOrganizer default 
		packageNamed: 'Unclassified'
		ifAbsent: [ self error ])).
	.
	self repository addAction: (ICUpdateClassAction for: self fakeClass).  
	self repository addAction: (ICUpdateClassAction for: Object). 
	self repository addAction: (ICUpdateClassAction for: Model). 
	self repository addAction: (ICUpdateMethodAction for: (MethodReference 
		class: SequenceableCollection 
		selector: #putOn:)).
	self repository push.
	self assert: (self repository storage / 'Unclassified' / 'FakeClass.st') exists. 
	self assert: (self repository storage / 'Kernel-Objects' / 'Object.st') exists. 
	self assert: (self repository storage / 'Kernel-Objects' / 'Model.st') exists. 
	self assert: (self repository storage / 'Extensions' / 'SequenceableCollection.st') exists. ! !

!ICOperationTest methodsFor: 'tests' stamp: 'EstebanLorenzano 1/17/2011 17:13'!
testPushUpdateClass
	self repository containers add: (ICPackageContainer for: (PackageOrganizer default 
		packageNamed: 'Unclassified'
		ifAbsent: [ self error ])).
	self repository addAction: (ICUpdateClassAction for: self fakeClass).  
	self repository push.
	
	self assert: (self repository storage / 'Unclassified' / 'FakeClass.st') exists. ! !

ICBaseTest subclass: #ICRepositoryTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ice-Tests'!

!ICRepositoryTest methodsFor: 'tests' stamp: 'EstebanLorenzano 1/19/2011 15:57'!
testAddActionAddClass
	| newAction |
	
	self repository addPackageNamed: 'Unclassified'.
	
	newAction := ICAddClassAction for: self fakeClass.
	self repository addAction: newAction.
	
	self assert: self repository isDirty.
	self assert: self repository containersWithActions size = 1.
	self assert: (self repository actions includes: newAction).
	! !

!ICRepositoryTest methodsFor: 'tests' stamp: 'EstebanLorenzano 1/19/2011 15:57'!
testAddActionAddMethod
	| newAction |
	
	self repository addPackageNamed: 'Unclassified'.
	
	newAction := ICAddMethodAction for: (MethodReference 
		class: self fakeClass
		selector: #fakeMethod).
	self repository addAction: newAction.
	self assert: self repository isDirty.
	self assert: self repository containersWithActions size = 1.
	self assert: (self repository actions includes: newAction).
	! !

!ICRepositoryTest methodsFor: 'tests' stamp: 'EstebanLorenzano 1/19/2011 15:57'!
testAddActionRemoveClass
	| newAction |
	
	self repository addPackageNamed: 'Unclassified'.   
	
	newAction := ICRemoveClassAction for: self fakeClass.
	self repository addAction: newAction.	
	
	self assert: self repository isDirty.
	self assert: self repository containersWithActions size = 1.
	self assert: (self repository actions includes: newAction).! !

!ICRepositoryTest methodsFor: 'tests' stamp: 'EstebanLorenzano 1/19/2011 15:57'!
testAddActionRemoveMethod
	| newAction |
	
	self repository addPackageNamed: 'Unclassified'.   
	
	newAction := ICRemoveMethodAction for: (MethodReference 
		class: self fakeClass
		selector: #fakeMethod).
	self repository addAction: newAction.
	
	self assert: self repository isDirty.
	self assert: self repository containersWithActions size = 1.
	self assert: (self repository actions includes: newAction).! !

!ICRepositoryTest methodsFor: 'tests' stamp: 'EstebanLorenzano 1/19/2011 15:57'!
testAddActionUpdateAfterAddClass
	| newAction1 newAction2 |
	
	self repository addPackageNamed: 'Unclassified'.   
	
	newAction1 := ICAddClassAction for: self fakeClass.
	newAction2 := ICUpdateClassAction for: self fakeClass.
	self repository addAction: newAction1.
	self repository addAction: newAction2.
	
	self assert: self repository isDirty.
	self assert: self repository containersWithActions size = 1.
	self assert: (self repository actions includesAll: {newAction1. newAction2}).! !

!ICRepositoryTest methodsFor: 'tests' stamp: 'EstebanLorenzano 1/19/2011 15:57'!
testAddActionUpdateAfterAddMethod
	| newAction1 newAction2 |
	
	self repository addPackageNamed: 'Unclassified'.   

	newAction1 := ICAddMethodAction for: (MethodReference 
		class: self fakeClass
		selector: #fakeMethod).
	newAction2 := ICUpdateMethodAction for: (MethodReference 
		class: self fakeClass
		selector: #fakeMethod).	
	self repository addAction: newAction1.	
	self repository addAction: newAction2.
	
	self assert: self repository isDirty.
	self assert: self repository containersWithActions size = 1.
	self assert: (self repository actions includesAll: {newAction1. newAction2}).
	! !

!ICRepositoryTest methodsFor: 'tests' stamp: 'EstebanLorenzano 1/19/2011 15:57'!
testAddActionUpdateClass
	| newAction |
	
	self repository addPackageNamed: 'Unclassified'.   
	
	newAction := ICUpdateClassAction for: self fakeClass.
	self repository addAction: newAction.
	
	self assert: self repository isDirty.
	self assert: self repository containersWithActions size = 1.
	self assert: (self repository actions includes: newAction).
	! !

!ICRepositoryTest methodsFor: 'tests' stamp: 'EstebanLorenzano 1/19/2011 15:57'!
testAddActionUpdateMethod
	| newAction |
	self repository addPackageNamed: 'Unclassified'.   
	
	newAction := ICUpdateMethodAction for: (MethodReference 
		class: self fakeClass
		selector: #fakeMethod).
	self repository addAction: newAction.
	
	self assert: self repository isDirty.
	self assert: self repository containersWithActions size = 1.
	self assert: (self repository actions includes: newAction).
	! !

!ICRepositoryTest methodsFor: 'tests' stamp: 'EstebanLorenzano 1/19/2011 15:57'!
testAddExtensions 
	self repository addPackageNamed: 'FakeExtension'. 
	self assert: self repository containersWithActions size = 1.
	self assert: self repository extensionContainer allActions size = 1.
	self assert: self repository extensionContainer allActions first name = #fakeMethod.
	self assert: self repository extensionContainer allActions first className = #FakeClass.! !

!ICRepositoryTest methodsFor: 'tests' stamp: 'EstebanLorenzano 1/19/2011 15:57'!
testAddExtensionsThenAddPackage 
	self repository addPackageNamed: 'FakeExtension'. 
	self repository addPackageNamed: 'Unclassified'. 
	
	self assert: self repository extensionContainer allActions isEmpty. 
! !

!ICRepositoryTest methodsFor: 'tests' stamp: 'EstebanLorenzano 1/19/2011 15:57'!
testAddPackage 
	| fakeClassToAdd |
	
	fakeClassToAdd := Class subclass: #FakeClassToAdd. 
	[ 
		self repository addPackageNamed: 'Unclassified'. 
		self assert: self repository containersWithActions size = 1.
		self assert: (self repository actions 
			anySatisfy: [ :each | 
				(each className = 'FakeClassToAdd') and: [ each class = ICAddClassAction ] ])
	] ensure: [ fakeClassToAdd removeFromSystem: false ]! !

!ICRepositoryTest methodsFor: 'tests' stamp: 'EstebanLorenzano 1/19/2011 15:57'!
testAddPackageThenAddExtensions 
	self repository addPackageNamed: 'Unclassified'. 
	self repository addPackageNamed: 'FakeExtension'. 
	
	self assert: self repository extensionContainer allActions isEmpty. 
! !

!ICRepositoryTest methodsFor: 'tests' stamp: 'EstebanLorenzano 1/19/2011 15:57'!
testChangesAddAction
	| changes fakeClass2 |
	self repository addPackageNamed:  'Unclassified'.
	self repository addAction: (ICUpdateClassAction for: self fakeClass).

	"just one change (one class)"	
	changes := self repository changes.
	self assert: changes notEmpty.
	self assert: changes size = 1.
	self assert: changes first class = ICUpdateClassChange.
	self assert: changes first className = #FakeClass.
	self assert: changes first classCategory = 'Unclassified'.
	
	self repository addAction: (ICUpdateMethodAction for: (MethodReference 
		class: self fakeClass
		selector: #fakeMethod)).  

	"still just one change (one class)"
	changes := self repository changes.
	self assert: changes notEmpty.
	self assert: changes size = 1.
	self assert: changes first class = ICUpdateClassChange.
	self assert: changes first className = #FakeClass.
	self assert: changes first classCategory = 'Unclassified'.

	fakeClass2 := Class subclass: #FakeClass2.
	[ 
		self repository addAction: (ICAddClassAction for: fakeClass2).

		"Now, two changes (two classes)"
		changes := self repository changes.
		self assert: changes notEmpty.
		self assert: changes size = 2.
		self assert: changes last class = ICUpdateClassChange.
		self assert: changes last className = #FakeClass2.
		self assert: changes last classCategory = 'Unclassified'. ]
	ensure: [ fakeClass2 removeFromSystem: false ]  ! !

!ICRepositoryTest methodsFor: 'tests' stamp: 'EstebanLorenzano 1/19/2011 15:57'!
testChangesAddContainer
	| changes |
	
	self repository addPackageNamed: 'FakeExtension'.
	"One change (one class with extensions)"  
	changes := self repository changes.
	self assert: changes notEmpty.
	self assert: changes size = 1.
	self assert: changes first class = ICUpdateExtensionsChange.
	self assert: changes first className = #FakeClass.
	self assert: changes first classCategory = 'Extensions'.	
	
	self repository addPackageNamed:  'Unclassified'.
	"One change (one class, no extensions)"
	changes := self repository changes.
	self assert: changes notEmpty.
	self assert: changes size = 1.
	self assert: changes first class = ICUpdateClassChange.
	self assert: changes first className = #FakeClass.
	self assert: changes first classCategory = 'Unclassified'.! !

!ICRepositoryTest methodsFor: 'tests' stamp: 'DeboraFortini 9/6/2011 17:24'!
testStatusOk
	"FSMemoryFilesystem fails to give me valid modification times, I need to use a real repository to 
	 test this"
	| fileRepository reference |

	reference := FSFilesystem disk workingDirectory  / 'repository-test'.
	reference ensureDirectory.
	fileRepository := ICRepository on: reference.
	
	[
		"Just added, FakeClass status is 'Changed'"
		fileRepository addPackageNamed: 'Unclassified'.  
		self assert: fileRepository status = ICRepository statusChanged.
	
		"After push repository is 'Ok'"
		fileRepository push.
		self assert: fileRepository status = ICRepository statusOk.
	
		"After added action, repository is 'Changed'"
		fileRepository addAction: (ICUpdateClassAction for: self fakeClass).
		self assert: fileRepository status = ICRepository statusChanged.
		
		"After modify externally smalltalk file, with internal change, repository is 'Conflict'"
		(Delay forSeconds: 1) wait. "Ensure change time (since nanoseconds is not working for now :()"
		(reference / 'test.st') fileStreamDo: [ :stream | stream nextPutAll: 'testing' ].
		self assert: fileRepository status = ICRepository statusConflict ]	
	ensure: [ reference deleteAll ]
	! !
